@@ -2,7 +2,7 @@ package Resource::Silo::Container;
 
 use strict;
 use warnings;
-our $VERSION = '0.1200_01';
+our $VERSION = '0.01';
 
 =head1 NAME
 
@@ -23,11 +23,8 @@ as well as a doorway into a fine-grained control interface.
 
 =cut
 
-use Moo;
 use Carp;
 use Scalar::Util qw( blessed refaddr reftype weaken );
-use Module::Load qw( load );
-use namespace::clean;
 
 my $ID_REX = qr/^[a-z][a-z_0-9]*$/i;
 
@@ -44,23 +41,27 @@ L</override> method (see below).
 # please make sure all internal fields start with a hyphen ("-").
 
 my %active_instances;
+my $not_once = \%Resource::Silo::metadata; # avoid once warning
 
-sub BUILD {
-    my ($self, $args) = @_;
+sub new {
+    my $class = shift;
+    $class = ref $class if blessed $class;
 
-    my $spec = $Resource::Silo::metadata{ref $self}
-        // _silo_find_metaclass($self);
-
-    $self->{-spec} = $spec;
-    $self->{-pid} = $$;
-
-    $self->_silo_do_override($args);
+    my $spec = $Resource::Silo::metadata{$class}
+        or croak "Failed to locate \$Resource::Silo::metadata for class $class";
 
+    my $self = bless {
+        -pid  => $$,
+        -spec => $spec,
+    }, $class;
+    $self->ctl->override( @_ )
+        if @_;
     $active_instances{ refaddr $self } = $self;
     weaken $active_instances{ refaddr $self };
+    return $self;
 };
 
-sub DEMOLISH {
+sub DESTROY {
     my $self = shift;
     delete $active_instances{ refaddr $self };
     $self->ctl->cleanup;
@@ -75,51 +76,15 @@ END {
     };
 };
 
-=head2 C<ctl>
-
-As the container class may contain arbitrary resource names and
-user-defined methods to boot, we intend to declare as few public methods
-as possible.
-
-Instead, we create a facade object that has access to container's internals
-and can perform fine-grained management operations.
-See L</CONTROL INTERFACE> below.
-
-Example:
-
-    # Somewhere in a test file
-    use Test::More;
-    use My::App qw(silo);
-
-    silo->ctl->override( dbh => $fake_database_connection );
-    silo->ctl->lock; # forbid instantiating new resources
-
-Returns a facade object.
-
-B<NOTE> Such object contains a weak reference to the parent object
-and thus must not be saved anywhere, lest you be surprised.
-Use it and discard immediately.
-
-=cut
-
-sub ctl {
-    my $self = shift;
-    my $facade = bless \$self, 'Resource::Silo::Container::Dashboard';
-    weaken $$facade;
-    confess "Attempt to close over nonexistent value"
-        unless $$facade;
-    return $facade;
-};
-
 # Instantiate resource $name with argument $argument.
 # This is what a silo->resource_name calls after checking the cache.
-sub _silo_instantiate_res {
+sub _instantiate_resource {
     my ($self, $name, $arg) = @_;
 
     croak "Illegal resource name '$name'"
         unless $name =~ $ID_REX;
 
-    my $spec = $self->{-spec}{resource}{$name};
+    my $spec = $self->{-spec}->spec($name);
     $arg //= '';
 
     croak "Attempting to fetch nonexistent resource '$name'"
@@ -133,48 +98,36 @@ sub _silo_instantiate_res {
         if $self->{-cleanup};
     croak "Attempting to initialize resource '$name' in locked mode"
         if $self->{-locked}
-            and !$spec->{derived}
+            and !$spec->{derivative}
             and !$self->{-override}{$name};
 
-    self->_silo_unexpected_dep($name)
+    croak "Attempting to fetch unexpected dependency '$name'"
         if ($self->{-allow} && !$self->{-allow}{$name});
+    local $self->{-allow} = $spec->{allowdeps};
 
     # Detect circular dependencies
-    my $key = $name . (length $arg ? "/$arg" : '');
+    my $key = $name . (length $arg ? "\@$arg" : '');
     if ($self->{-pending}{$key}) {
         my $loop = join ', ', sort keys %{ $self->{-pending} };
         croak "Circular dependency detected for resource $key: {$loop}";
     };
-
-    # Try loading modules
-    foreach my $mod (@{ $spec->{require} }) {
-        eval { load $mod; 1 }
-            or croak "resource '$name': failed to load '$mod': $@";
-    };
-
-    # Finally set the temporary flags
-    local $self->{-onbehalf} = $name; # should we use a stack instead?
     local $self->{-pending}{$key} = 1;
-    local $self->{-allow} = $spec->{allowdeps};
 
-    ($self->{-override}{$name} // $spec->{init})->($self, $name, $arg)
-        // croak "Instantiating resource '$key' $spec->{origin} failed for no apparent reason";
+    ($self->{-override}{$name} // $spec->{init})->($self, $name, $arg);
 };
 
 # use instead of delete $self->{-cache}{$name}
-sub _silo_cleanup_res {
+sub _cleanup_resource {
     my ($self, $name, @list) = @_;
-
     # TODO Do we need to validate arguments here?
-    my $spec = $self->{-spec}{resource}{$name};
 
     my $action;
     if (!$self->{-override}{$name}) {
         # 1) skip resources that have overrides
         # 2) if we're in "no pid" mode, use fork_cleanup if available
         $action = $self->{-pid} != $$
-            && $spec->{fork_cleanup}
-            || $spec->{cleanup};
+            && $self->{-spec}{$name}{fork_cleanup}
+            || $self->{-spec}{$name}{cleanup};
     };
     my $known = $self->{-cache}{$name};
 
@@ -191,14 +144,14 @@ sub _silo_cleanup_res {
 
 # We must create resource accessors in this package
 #   so that errors get attributed correctly
-#   (+ This way no other classes need to know our internal structure)
-sub _silo_make_accessor {
+#   (+ This way no other class need to know our internal structure)
+sub _make_resource_accessor {
     my ($name, $spec) = @_;
 
     if ($spec->{ignore_cache}) {
         return sub {
             my ($self, $arg) = @_;
-            return $self->_silo_instantiate_res($name, $arg);
+            return $self->_instantiate_resource($name, $arg);
         };
     };
 
@@ -211,84 +164,53 @@ sub _silo_make_accessor {
             $self->{-pid} = $$;
         };
 
-        # We must check dependencies even before going to the cache
-        $self->_silo_unexpected_dep($name)
+        croak "Attempting to fetch unexpected dependency '$name'"
             if ($self->{-allow} && !$self->{-allow}{$name});
 
-        # Stringify $arg ASAP, we'll validate it inside _silo_instantiate_res().
+        # Stringify $arg ASAP, we'll validate it inside _instantiate_resource().
         # The cache entry for an invalid argument will never get populated.
         my $key = defined $arg && !ref $arg ? $arg : '';
-        $self->{-cache}{$name}{$key} //= $self->_silo_instantiate_res($name, $arg);
+        $self->{-cache}{$name}{$key} //= $self->_instantiate_resource($name, $arg);
     };
 };
 
-sub _silo_check_overrides {
-    my ($self, $subst) = @_;
-
-    my $known = $self->{-spec}{resource};
-    my @bad = grep { !$known->{$_} } keys %$subst;
-    croak "Attempt to override unknown resource(s): "
-        .join ", ", map { "'$_'" } @bad
-            if @bad;
-};
-
-sub _silo_do_override {
-    my ($self, $subst) = @_;
-
-    my $known = $self->{-spec}{resource};
-
-    foreach my $name (keys %$subst) {
-        # Just skip over unknown resources if we're in constructor
-        next unless $known->{$name};
-        my $init = $subst->{$name};
-
-        # Finalize existing values in cache, just in case
-        # BEFORE setting up override
-        $self->_silo_cleanup_res($name);
-
-        if (defined $init) {
-            $self->{-override}{$name} = (reftype $init // '') eq 'CODE'
-                ? $init
-                : sub { $init };
-        } else {
-            delete $self->{-override}{$name};
-        };
-    };
-}
-
-sub _silo_unexpected_dep {
-    my ($self, $name) = @_;
-    my $spec = $self->{-spec}{resource}{$name};
-
-    my $explain = $spec->{autodeps}
-        ? ". Use explicit 'dependencies' or the 'loose_deps' flag"
-        : " but is not listed in its dependencies";
-    croak "Resource '$name' was unexpectedly required by"
-        ." '$self->{-onbehalf}'$explain";
-}
-
-sub _silo_find_metaclass {
-    my $self = shift;
-    my $class = ref $self;
-
-    my @queue = $class;
-    while (defined( my $next = shift @queue )) {
-        my $meta = $Resource::Silo::metadata{$next};
-        return $meta if $meta;
-        no strict 'refs'; ## no critic strictures
-        push @queue, @{ "${next}::ISA" };
-    };
-
-    croak "Failed to locate \$Resource::Silo::metadata for class $class";
-};
-
 =head1 CONTROL INTERFACE
 
-The below methods are all accessible via
-C<$container-E<gt>ctl-E<gt>$method_name>.
+Sometimes more fine-grained control over the container is needed than just
+fetching the resources.
+
+Since the container class can contain arbitrary resource names and
+some user-defined methods to boot, instead of polluting its namespace,
+we provide a single method, C<ctl>, that returns a temporary facade object
+that provide access to the following methods.
+
+Most of them return the facade so that they can be chained. E.g.
+
+    # somewhere in your tests
+    silo->ctl->lock->override(
+        config  => { ... },     # some fixed values
+        dbh     => sub { ... }, # return SQLite instance
+    );
+
+=head2 C<ctl>
+
+Returns the facade.
+
+B<NOTE> the facade object is weak reference to the parent object
+and thus must not be save anywhere, lest you be surprised.
+Use it and discard immediately.
 
 =cut
 
+sub ctl {
+    my $self = shift;
+    my $facade = bless \$self, 'Resource::Silo::Container::Dashboard';
+    weaken $$facade;
+    confess "Attempt to close over nonexistent value"
+        unless $$facade;
+    return $facade;
+};
+
 # We're declaring a different package in the same file because
 # 1) it must have access to the internals anyway and
 # 2) we want to keep the documentation close to the implementation.
@@ -325,8 +247,25 @@ for the affected resources.
 sub override {
     my ($self, %subst) = @_;
 
-    $$self->_silo_check_overrides(\%subst);
-    $$self->_silo_do_override(\%subst);
+    foreach my $name (keys %subst) {
+        croak "Illegal resource name '$name'"
+            unless $name =~ $ID_REX;
+        croak "Attempt to override unknown resource '$name'"
+            unless $$self->{-spec}{$name};
+        my $init = $subst{$name};
+
+        # Finalize existing values in cache, just in case
+        # BEFORE setting up override
+        $$self->_cleanup_resource($name);
+
+        if (defined $init) {
+            $$self->{-override}{$name} = (reftype $init // '') eq 'CODE'
+                ? $init
+                : sub { $init };
+        } else {
+            delete $$self->{-override}{$name};
+        };
+    };
 
     return $self;
 }
@@ -336,7 +275,7 @@ sub override {
 Forbid initializing new resources.
 
 The cached ones instantiated so far, the ones that have been overridden,
-and the ones with the C<derived> flag will still be returned.
+and the ones with the C<derivative> flag will still be returned.
 
 =cut
 
@@ -365,21 +304,14 @@ Try loading all the resources that have C<preload> flag set.
 May be useful if e.g. a server-side application is starting and must
 check its database connection(s) before it starts handling any clients.
 
-In addition, self-check will be called and all declared C<require>'d
-modules will be loaded, even if they are not required by preloaded resources.
-
 =cut
 
 sub preload {
     my $self = shift;
     # TODO allow specifying resources to load
-    #      but first come up with a way to specify arguments, too.
+    #      but first come up with a way of specifying arguments, too.
 
-    my $meta = $$self->{-spec};
-
-    $meta->self_check;
-
-    my $list = $meta->{preload};
+    my $list = $$self->{-spec}{-preload};
     for my $name (@$list) {
         my $unused = $$self->$name;
     };
@@ -401,18 +333,17 @@ sub cleanup {
 
     # NOTE Be careful! cleanup must never ever die!
 
-    my $spec = $self->{-spec}{resource};
+    my $spec = $self->{-spec};
     my @order = sort {
         $spec->{$a}{cleanup_order} <=> $spec->{$b}{cleanup_order};
     } keys %{ $self->{-cache} };
 
     foreach my $name (@order) {
-        local $@; # don't pollute $@ if we're in destructor after an exception
         eval {
             # We cannot afford to die here as if we do
             #    a resource that causes exceptions in cleanup
             #    would be stuck in cache forever
-            $self->_silo_cleanup_res($name);
+            $self->_cleanup_resource($name);
             1;
         } or do {
             my $err = $@;
@@ -440,37 +371,7 @@ architectural problem.
 =cut
 
 sub fresh {
-    return ${+shift}->_silo_instantiate_res(@_);
-};
-
-=head2 list_cached
-
-Return list of services that are currently present in the cache as strings
-of form C<$service_name> or C<$service_name/$argument> if argument is present.
-
-Useful for debugging.
-
-=cut
-
-sub list_cached {
-    my $cache = ${+shift}->{-cache};
-    my @out;
-    foreach my $service (sort keys %$cache) {
-        foreach my $arg (sort keys %{ $cache->{$service} }) {
-            push @out, length $arg ? "$service/$arg" : $service;
-        };
-    };
-    return wantarray ? @out : \@out;
-};
-
-=head2 meta
-
-Get resource metadata object (a L<Resource::Silo::Metadata>).
-
-=cut
-
-sub meta {
-    return ${+shift}->{-spec};
+    return ${+shift}->_instantiate_resource(@_);
 };
 
 =head1 COPYRIGHT AND LICENSE
